package com.persagy.apm.common.model.query;

import org.apache.commons.collections4.CollectionUtils;

import java.util.ArrayList;
import java.util.List;

/**
 * 查询条件逻辑表达式表达式
 * @author Sun
 * @version 1.0 2014年9月24日
 * @since 1.6
 */
public class ChainLogicExpression implements IExpression {

	/**
	 * 序列id
	 */
	private static final long serialVersionUID = 5682864471973760964L;

	/**
	 * 表达式列表
	 */
	private List<IExpression> expressions = null;
	
	/**
	 * 操作符
	 */
	private boolean isOr = false;
	
	/**
	 * 构造方法
	 */
	public ChainLogicExpression(){
		
	}
	
	/**
	 * 表达式构造方法
	 * @param isOr       是否为逻辑或相连
	 * @param expressions　表达式数组
	 */
	public ChainLogicExpression(boolean isOr, IExpression...expressions){
		addExpression(expressions);
		setOr(isOr);
	}

	/**
	 * 取得链式逻辑表达式中包含的表达式列表
	 * @return 表达式列表
	 */
	public List<IExpression> getExpressions() {
		return expressions;
	}

	/**
	 * 设置链式逻辑表达式中包含的表达式列表
	 * @param expressions 表达式列表
	 */
	public void setExpressions(List<IExpression> expressions) {
		this.expressions = expressions;
	}
	
	/**
	 * 添加表达式到链式表达式中
	 * @param expressions 表达式数组
	 */
	public void addExpression(IExpression... expressions){
		if(this.expressions == null) {
            this.expressions = new ArrayList<>();
        }
		CollectionUtils.addAll(this.expressions, expressions);
	}
	
	/**
	 * 判断是否为‘或’逻辑操作
	 * @return 布尔值
	 */
	public boolean isOr() {
		return isOr;
	}

	/**
	 * 设置是否为‘或’逻辑操作
	 * @param isOr 布尔值
	 */
	public void setOr(boolean isOr) {
		this.isOr = isOr;
	}

	@Override
	public ExpressionType getType() {
		return ExpressionType.CHAIN;
	}

	/**
	 * 取得表达式大小
	 * @return 表达式大小
	 */
	public int size(){
		if(expressions == null) {
			return 0;
		}
		return expressions.size();
	}
}
