package com.persagy.apm.common.service;

import com.persagy.apm.common.constant.CommonConstant;
import com.persagy.apm.common.utils.FileUtil;
import lombok.extern.slf4j.Slf4j;
import org.apache.poi.ss.usermodel.Workbook;
import org.apache.poi.ss.usermodel.WorkbookFactory;
import org.springframework.core.io.Resource;

import javax.servlet.http.HttpServletResponse;
import java.io.IOException;
import java.io.InputStream;
import java.net.URLEncoder;
/***
 * Description: 下载Excel的公共逻辑抽象类
 * @author : lijie
 * @date :2021/4/28 17:22
 * Update By lijie 2021/4/28 17:22
 */
@Slf4j
public abstract class DownloadService {

    private static final String CONFIG_PATH_PREFIX="file:./config";
    private static final String CLASS_PATH_PREFIX="classpath:.";

    /***
     * Description: 公共下载方法的壳子
     * @return : void
     * @author : lijie
     * @date :2021/4/28 16:55
     * Update By lijie 2021/4/28 16:55
     */
    public void downloadFile() throws IOException {
        String filePath = getFilePath();
        Resource[] resolverResources = FileUtil.getResource(CONFIG_PATH_PREFIX+filePath, CLASS_PATH_PREFIX+filePath);
        if (resolverResources.length<=0){
            return;
        }
        Resource resolverResource = resolverResources[0];
        String filename = resolverResource.getFilename();
        InputStream inputStream = resolverResource.getInputStream();
        Workbook workbook = null;
        try {
            workbook = WorkbookFactory.create(inputStream);
            handleWorkbook(workbook);
            HttpServletResponse response = getHttpServletResponse();
            // 为文件命名
            response.setHeader(CommonConstant.REPONSE_HEAD_CONTENT_DISPOSITION,
                    CommonConstant.REPONSE_HEAD_CONTENT_DISPOSITION_ATTACHMENT + URLEncoder.encode(filename, "UTF-8"));
            response.addHeader(CommonConstant.REPONSE_HEAD_CONTENT_TYPE, CommonConstant.REPONSE_HEAD_CONTENT_TYPE_SHEET);
            workbook.write(response.getOutputStream());
        } catch (Exception e) {
            log.error(e.getMessage());
        } finally {
            try {
                inputStream.close();
            } catch (IOException ignored) {
                log.error("关闭文件流失败");
            }
            if (workbook != null) {
                try {
                    workbook.close();
                } catch (IOException ignored) {
                    log.error("关闭工作簿失败");
                }
            }
        }
    }
    /***
     * Description: 核心处理业务逻辑的方法
     * @param workbook : 工作簿
     * @return : void
     * @author : lijie
     * @date :2021/4/28 17:20
     * Update By lijie 2021/4/28 17:20
     */
    protected abstract void handleWorkbook(Workbook workbook);
    /***
     * Description: 获取文件路径
     * @return : java.lang.String
     * @author : lijie
     * @date :2021/4/28 17:21
     * Update By lijie 2021/4/28 17:21
     */
    protected abstract String getFilePath();
    /***
     * Description: 获取响应对象
     * @return : javax.servlet.http.HttpServletResponse
     * @author : lijie
     * @date :2021/4/28 17:21
     * Update By lijie 2021/4/28 17:21
     */
    protected abstract HttpServletResponse getHttpServletResponse();

}
