package com.persagy.apm.translate.extension;

import org.apache.ibatis.session.ResultContext;
import org.apache.ibatis.session.ResultHandler;

import java.util.HashMap;
import java.util.Map;

/**
 * 基于两列生成map的结果处理，会把其中一列设置为key,另一列设置为value
 * @author Sun
 * @version 1.0 2015-7-23
 * @since 1.6
 */
public class MapPairResultHandler implements ResultHandler {

    /**
     * 键值字段
     */
    private String mapKey = null;

    /**
     * 映射值字段
     */
    private String mapValue = null;

    /**
     * 结果map
     */
    private final Map<String, Object> mappedResults;


    /**
     * 构造方法
     * @param mapKey 作为键值的字段名
     */
    public MapPairResultHandler(String mapKey, String mapValue){
        this.mapKey = mapKey;
        this.mapValue = mapValue;
        mappedResults = new HashMap<>();
    }

    /**
     * 构造方法
     * @param mapKey 作为键值的字段名
     * @param map 已有的map对象
     */
    public MapPairResultHandler(String mapKey, String mapValue, Map<String,Object> map){
        this.mapKey = mapKey;
        this.mapValue = mapValue;
        mappedResults = map;
    }

    @Override
    public void handleResult(ResultContext context) {
        @SuppressWarnings("unchecked")
        Map<String,String> row =(Map)context.getResultObject();
        String key = row.get(mapKey);
        mappedResults.put(key,row.get(mapValue));
    }

    /**
     * 取得结果map对象
     * @return 结果map对象
     */
    public Map<String,Object> getResult(){
        return mappedResults;
    }


}
