package com.persagy.apm.translate.extension;

import org.apache.ibatis.session.ResultContext;
import org.apache.ibatis.session.ResultHandler;

import java.util.HashMap;
import java.util.Map;

/**
 * 基于map结果处理这里因为mapkey为动态的所以自行处理
 * @author Sun
 * @version 1.0 2015-7-23
 * @since 1.6
 */
public class MapResultHandler<T> implements ResultHandler {

    /**
     * 键值字段
     */
    private String mapKey = null;


    /**
     * 结果map
     */
    private final Map<String, Object> mappedResults;


    /**
     * 构造方法
     * @param mapKey 作为键值的字段名
     */
    public MapResultHandler(String mapKey){
        this.mapKey = mapKey;
        mappedResults = new HashMap<>();
    }

    /**
     * 构造方法
     * @param mapKey 作为键值的字段名
     * @param map 已有的map对象
     */
    public MapResultHandler(String mapKey, String mapValue, Map<String,Object> map){
        this.mapKey = mapKey;
        mappedResults = map;
    }

    @Override
    public void handleResult(ResultContext context) {
        @SuppressWarnings("unchecked")
        Map<String,Object> row =(Map)context.getResultObject();
        String key = (String) row.get(mapKey);
        mappedResults.put(key,row);
    }

    /**
     * 取得结果map对象
     * @return 结果map对象
     */
    public Map<String,Object> getResult(){
        return mappedResults;
    }


}
