package com.persagy.apm.translate.impl;

import cn.hutool.core.collection.CollUtil;
import com.persagy.common.exception.BusinessException;
import com.persagy.apm.common.helper.SpringHelper;
import com.persagy.apm.translate.model.ITransWrapper;
import com.persagy.apm.translate.model.TransMeta;
import com.persagy.apm.translate.model.TransType;
import com.persagy.apm.translate.service.ITranslateBatchHandle;
import com.persagy.apm.translate.service.ITranslator;
import org.apache.commons.lang3.StringUtils;
import org.springframework.context.annotation.Scope;
import org.springframework.stereotype.Service;

import java.util.Collection;
import java.util.List;
import java.util.Map;


/**
 * 基于指定服务接口的数据翻译
 * @author Charlie Yu
 * @Date 2021-03-03
 */
@SuppressWarnings("ALL")
@Service(TransType.SERVICE)
@Scope("prototype")
public class ServiceTranslateHandleImpl implements ITranslateBatchHandle {

	/**
	 * 翻译服务对象
	 */
	private ITranslator translator;
	
	/**
	 * 翻译元数据
	 */
	private TransMeta meta;
	
	/**
	 * 目标字段
	 */
	private String[] targetFields = null;

    @Override
	public TransMeta getMeta() {
		return meta;
	}

    @Override
	public void setMeta(TransMeta meta) {
		this.meta = meta;
	}

	@Override
	public ITransWrapper trans(ITransWrapper data) {
		if(translator == null){
			translator = SpringHelper.getBean(meta.getSource(), ITranslator.class);
			if(translator == null){
				throw new BusinessException("翻译服务配置出错");
			}
			String target = meta.getTarget();
			if(StringUtils.isEmpty(target)){
				throw new BusinessException("翻译服务配置出错");
			}
			targetFields = meta.getTarget().split(",");
		}
		Object parameter = data.getValue(meta.getParameter());
		if(parameter instanceof Collection) {
			parameter = CollUtil.join(((Collection)parameter), ",");
		}
		String rsStr = translator.trans(parameter.toString());
		String[] rs = null;
		if(StringUtils.isNotEmpty(rsStr)){
			rs = rsStr.split(",",targetFields.length);
		}
		for(int i = 0; i < targetFields.length; i++){
			if(rs == null || rs.length <= i){
				data.setValue(targetFields[i], null);
			}else{
				data.setValue(targetFields[i], rs[i]);
			}
		}
		return data;
	}

	@Override
	public Map<String, Object> batchTrans(List<String> parameter) {
		if(translator == null){
			translator = SpringHelper.getBean(meta.getSource(),ITranslator.class);
			if(translator == null){
				throw new BusinessException("翻译服务配置出错");
			}
		}
		return translator.transBatch(parameter);
	}

}
